@@ -3,7 +3,9 @@
 #include "liblzma_util.h"
 
 PyDoc_STRVAR(LZMAComp_compress__doc__,
-"compress(data) -- Feed the compressor object with data to compress sequently.\n\
+"compress(data) -> string\n\
+\n\
+Feed the compressor object with data to compress sequently.\n\
 This function will return the header for the compressed string for the first\n\
 input provided, this header will be needed to concatenate with the rest of\n\
 the stream when flushing to have a proper stream able to be decompressed\n\
@@ -50,7 +52,7 @@ LZMAComp_compress(LZMACompObject *self, PyObject *args)
 		lzus->next_out = (uint8_t *)PyString_AS_STRING(RetVal) + length;
 		lzus->avail_out = (size_t)length;
 		length = length << 1;
-		if(!Util_CatchLZMAError(lzuerror, lzus))
+		if(!Util_CatchLZMAError(lzuerror, lzus, true))
 			goto error;
 	}
 
@@ -66,9 +68,11 @@ LZMAComp_compress(LZMACompObject *self, PyObject *args)
 }
 
 PyDoc_STRVAR(LZMAComp_flush__doc__,
-"flush( [mode] ) -- Return a string containing any remaining compressed data.\n\
+"flush( [mode] ) -> string\n\
 \n\
-mode can be one of the constants LZMA_SYNC_FLUSH, LZMA_FULL_FLUSH, LZMA_FINISH; the\n\
+Returns a string containing any remaining compressed data.\n\
+\n\
+'mode' can be one of the constants LZMA_SYNC_FLUSH, LZMA_FULL_FLUSH, LZMA_FINISH; the\n\
 default value used when mode is not specified is LZMA_FINISH.\n\
 If mode == LZMA_FINISH, the compressor object can no longer be used after\n\
 calling the flush() method.  Otherwise, more data can still be compressed.\n");
@@ -78,7 +82,7 @@ LZMAComp_flush(LZMACompObject *self, PyObject *args)
 {
 	Py_ssize_t length = SMALLCHUNK;
     	PyObject * RetVal = NULL;
-	uint8_t flushmode = LZMA_FINISH;
+	lzma_action flushmode = LZMA_FINISH;
     	uint64_t start_total_out;
 	lzma_stream *lzus = &self->lzus;
 	lzma_ret lzuerror;
@@ -96,16 +100,21 @@ LZMAComp_flush(LZMACompObject *self, PyObject *args)
 	switch(flushmode){
 		case(LZMA_SYNC_FLUSH):
 		case(LZMA_FULL_FLUSH):
-			PyErr_Format(LZMAError, "%d is not supported as flush mode for LZMA_Alone format", flushmode);
-			goto error;
-			/* Flushing with LZMA_RUN is a no-op, so there's no point in
-			 * doing any work at all; just return an empty string.
-			 */
+			if(self->filters[0].id == LZMA_FILTER_LZMA1) {
+				PyErr_Format(LZMAError, "%d is not supported as flush mode for LZMA_Alone format", flushmode);
+				goto error;
+			}
+		/* Flushing with LZMA_RUN is a no-op, so there's no point in
+		 * doing any work at all; just return an empty string.
+		 */
 		case(LZMA_RUN):
 			RetVal = PyString_FromStringAndSize(NULL, 0);
 			goto error;
-		default:
+		case(LZMA_FINISH):
 			break;
+		default:
+			PyErr_Format(LZMAError, "Invalid flush mode: %d", flushmode);
+			goto error;
 	}
 
 	self->running = false;
@@ -128,7 +137,7 @@ LZMAComp_flush(LZMACompObject *self, PyObject *args)
 		lzus->next_out = (uint8_t *)PyString_AS_STRING(RetVal) + length;
 		lzus->avail_out = (size_t)length;
 		length = length << 1;
-		if(!Util_CatchLZMAError(lzuerror, lzus))
+		if(!Util_CatchLZMAError(lzuerror, lzus, true))
 			goto error;
     }
 
@@ -144,31 +153,42 @@ error:
 }
 
 PyDoc_STRVAR(LZMAComp_reset__doc__,
-"reset([format='alone', level=7, dictionary_size=23, literal_contextbits=3,\n\
-literal_pos_bits=0, pos_bits=2, mode=2, fast_bytes=128, match_finder='bt4',\n\
-match_finder_cycles=0]) -- Resets the compression object keeping the\n\
-compression settings. These existing settings can be overriden by providing\n\
+"reset(["DEFAULT_OPTIONS_STRING"]) -> None\n\
+\n\
+Resets the compression object keeping the compression settings.\n\
+These existing settings can be overriden by providing\n\
 keyword settings.");
 
 static PyObject *
-LZMAComp_reset(LZMACompObject *self, __attribute__((unused)) PyObject *args, PyObject *kwargs)
+LZMAComp_reset(LZMACompObject *self, PyObject *args, PyObject *kwargs)
 {
-	PyObject *result=NULL;
+	PyObject *result = NULL, *options_dict = NULL;
 	lzma_stream *lzus = &self->lzus;
-	lzma_ret lzuerror;
+	lzma_ret lzuerror = LZMA_OK;
+
+	static char *kwlist[] = {"options", NULL};
 
 	INITCHECK
-	if(!init_lzma_options("reset", kwargs, &self->options))
-		return NULL;
-	self->lzma_options = LZMA_options_get(self->options);
+	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|O:reset", kwlist,
+				&options_dict))
+		goto error;
+	
+	if(!init_lzma_options("reset", options_dict, self->filters))
+		goto error;
+
+	self->lzma_options = LZMA_options_get(self->filters[0]);
 
 
 	ACQUIRE_LOCK(self);
     	if (self->running)
 		lzma_end(lzus);
 	
-	lzuerror = lzma_alone_encoder(lzus, &self->options);
-	if(!Util_CatchLZMAError(lzuerror, lzus))
+	if(self->filters[0].id == LZMA_FILTER_LZMA2)
+		lzuerror = lzma_stream_encoder(lzus, self->filters, self->filters[LZMA_FILTERS_MAX + 1].id);
+	else if(self->filters[0].id == LZMA_FILTER_LZMA1)
+		lzuerror = lzma_alone_encoder(lzus, self->filters[0].options);
+
+	if(!Util_CatchLZMAError(lzuerror, lzus, true))
 		goto error;
 	self->running = true;
 
@@ -199,15 +219,23 @@ static PyMethodDef LZMAComp_methods[] =
     	{0, 0, 0, 0}
 };
 
-static int8_t
-LZMAComp_init(LZMACompObject *self, __attribute__((unused)) PyObject *args, PyObject *kwargs)
+static int
+LZMAComp_init(LZMACompObject *self, PyObject *args, PyObject *kwargs)
 {
+	PyObject *options_dict = NULL;
 	lzma_stream *lzus = &self->lzus;
-	lzma_ret lzuerror;
+	lzma_ret lzuerror = LZMA_OK;
 
-	if(!init_lzma_options("LZMACompressor", kwargs, &self->options))
+	static char *kwlist[] = {"options", NULL};
+
+	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|O:LZMACompressor", kwlist,
+				&options_dict))
+		return -1;
+	
+	if(!init_lzma_options("LZMACompressor", options_dict, self->filters))
 		goto error;
-	self->lzma_options = LZMA_options_get(self->options);
+
+	self->lzma_options = LZMA_options_get(self->filters[0]);
 
 #ifdef WITH_THREAD
 	self->lock = PyThread_allocate_lock();
@@ -217,8 +245,12 @@ LZMAComp_init(LZMACompObject *self, __attribute__((unused)) PyObject *args, PyOb
 	}
 #endif
 
-	lzuerror = lzma_alone_encoder(lzus, &self->options);
-	if(!Util_CatchLZMAError(lzuerror, lzus))
+	if(self->filters[0].id == LZMA_FILTER_LZMA2)
+		lzuerror = lzma_stream_encoder(lzus, self->filters, self->filters[LZMA_FILTERS_MAX + 1].id);
+	else if(self->filters[0].id == LZMA_FILTER_LZMA1)
+		lzuerror = lzma_alone_encoder(lzus, self->filters[0].options);
+
+	if(!Util_CatchLZMAError(lzuerror, lzus, true))
 		goto error;
 
 	self->is_initialised = true;
@@ -246,7 +278,7 @@ LZMACompObject_new(PyTypeObject *type, __attribute__((unused)) PyObject *args, _
 		self->running = false;
 		lzma_stream tmp = LZMA_STREAM_INIT;
 		self->lzus = tmp;
-		lzma_lzma_preset(&self->options, LZMA_DEFAULT_COMPRESSION-1);
+		self->filters[0].options = &self->options;
     	}
 	else
 		return NULL;
@@ -268,9 +300,7 @@ LZMAComp_dealloc(LZMACompObject *self)
 }
 
 PyDoc_STRVAR(LZMAComp__doc__,
-"LZMACompressor([format='alone', level=7, dictionary_size=23, literal_contextbits=3,\n\
-literal_pos_bits=0, pos_bits=2, mode=2, fast_bytes=128, match_finder='bt4',\n\
-match_finder_cycles=0]) -- Return a compressor object.\n\
+"LZMACompressor(["DEFAULT_OPTIONS_STRING"]) -> compressor object\n\
 Create a new compressor object. This object may be used to compress\n\
 data sequentially. If you want to compress data in one shot, use the\n\
 compress() function instead.\n");
@@ -322,5 +352,8 @@ PyTypeObject LZMAComp_Type = {
 	0,						/*tp_cache*/
 	0,						/*tp_subclasses*/
 	0,						/*tp_weaklist*/
-	0						/*tp_del*/
+	0,						/*tp_del*/
+#if PY_VERSION_HEX >= 0x02060000
+	0						/*tp_version_tag*/
+#endif
 };
