@@ -70,7 +70,7 @@ LZMAFile_read(LZMAFileObject *self, PyObject *args)
 			self->size = self->pos;
 			break;
 		} else if (lzuerror != LZMA_OK) {
-			Util_CatchLZMAError(lzuerror, &self->fp->strm);
+			Util_CatchLZMAError(lzuerror, &self->fp->strm, self->fp->encoding);
 			Py_DECREF(ret);
 			ret = NULL;
 			goto cleanup;
@@ -205,7 +205,7 @@ LZMAFile_readlines(LZMAFileObject *self, PyObject *args)
 			}
 			shortread = 1;
 		} else if (lzuerror != LZMA_OK) {
-			Util_CatchLZMAError(lzuerror, &self->fp->strm);
+			Util_CatchLZMAError(lzuerror, &self->fp->strm, self->fp->encoding);
 		  error:
 			Py_DECREF(list);
 			list = NULL;
@@ -315,7 +315,7 @@ LZMAFile_write(LZMAFileObject *self, PyObject *args)
 {
 	PyObject *ret = NULL;
 	char *buf;
-	int len;
+	Py_ssize_t len;
 	lzma_ret lzuerror;
 
 	if (!PyArg_ParseTuple(args, "s#:write", &buf, &len))
@@ -343,7 +343,7 @@ LZMAFile_write(LZMAFileObject *self, PyObject *args)
 	Py_END_ALLOW_THREADS
 
 	if (lzuerror != LZMA_OK) {
-		Util_CatchLZMAError(lzuerror, &self->fp->strm);
+		Util_CatchLZMAError(lzuerror, &self->fp->strm, self->fp->encoding);
 		goto cleanup;
 	}
 
@@ -466,7 +466,7 @@ LZMAFile_writelines(LZMAFileObject *self, PyObject *seq)
 				     PyString_AS_STRING(line), len);
 			if (lzuerror != LZMA_OK) {
 				Py_BLOCK_THREADS
-				Util_CatchLZMAError(lzuerror, &self->fp->strm);
+				Util_CatchLZMAError(lzuerror, &self->fp->strm, self->fp->encoding);
 				goto error;
 			}
 		}
@@ -508,7 +508,7 @@ LZMAFile_seek(LZMAFileObject *self, PyObject *args)
 	Py_off_t offset;
 	char small_buffer[SMALLCHUNK];
 	char *buffer = small_buffer;
-	size_t buffersize = SMALLCHUNK;
+	Py_ssize_t buffersize = SMALLCHUNK;
 	Py_off_t bytesread = 0;
 	size_t readsize;
 	int chunksize;
@@ -558,7 +558,7 @@ LZMAFile_seek(LZMAFileObject *self, PyObject *args)
 				if (lzuerror == LZMA_STREAM_END) {
 					break;
 				} else if (lzuerror != LZMA_OK) {
-					Util_CatchLZMAError(lzuerror, &self->fp->strm);
+					Util_CatchLZMAError(lzuerror, &self->fp->strm, self->fp->encoding);
 					goto cleanup;
 				}
 			}
@@ -579,9 +579,9 @@ LZMAFile_seek(LZMAFileObject *self, PyObject *args)
 	} else {
 		/* we cannot move back, so rewind the stream */
 		lzma_close_real(&lzuerror, self->fp);
-		lzma_open_real(&lzuerror, NULL, PyFile_AsFile(self->file));
+		lzma_open_real(&lzuerror, self->filters, PyFile_AsFile(self->file), self->memlimit);
 		if (lzuerror != LZMA_OK) {
-			Util_CatchLZMAError(lzuerror, &self->fp->strm);
+			Util_CatchLZMAError(lzuerror, &self->fp->strm, self->fp->encoding);
 			goto cleanup;
 		}
 		ret = PyObject_CallMethod(self->file, "seek", "(i)", 0);
@@ -590,9 +590,9 @@ LZMAFile_seek(LZMAFileObject *self, PyObject *args)
 		Py_DECREF(ret);
 		ret = NULL;
 		self->pos = 0;
-		self->fp = lzma_open_real(&lzuerror, NULL, PyFile_AsFile(self->file));
+		self->fp = lzma_open_real(&lzuerror, self->filters, PyFile_AsFile(self->file), self->memlimit);
 		if (lzuerror != LZMA_OK) {
-			Util_CatchLZMAError(lzuerror, &self->fp->strm);
+			Util_CatchLZMAError(lzuerror, &self->fp->strm, self->fp->encoding);
 			goto cleanup;
 		}
 	}
@@ -620,7 +620,7 @@ LZMAFile_seek(LZMAFileObject *self, PyObject *args)
 			self->size = self->pos;
 			break;
 		} else if (lzuerror != LZMA_OK) {
-			Util_CatchLZMAError(lzuerror, &self->fp->strm);
+			Util_CatchLZMAError(lzuerror, &self->fp->strm, self->fp->encoding);
 			goto cleanup;
 		}
 		if (bytesread == offset)
@@ -643,7 +643,7 @@ Return the current file position, an integer (may be a long integer).\n\
 ");
 
 static PyObject *
-LZMAFile_tell(LZMAFileObject *self, PyObject *args)
+LZMAFile_tell(LZMAFileObject *self, __attribute__((unused)) PyObject *args)
 {
 	PyObject *ret = NULL;
 
@@ -684,7 +684,7 @@ LZMAFile_close(LZMAFileObject *self)
 	}
 	ret = PyObject_CallMethod(self->file, "close", NULL);
 	if (lzuerror != LZMA_OK && lzuerror != LZMA_STREAM_END) {
-		Util_CatchLZMAError(lzuerror, NULL);
+		Util_CatchLZMAError(lzuerror, NULL, self->fp->encoding);
 		Py_XDECREF(ret);
 		ret = NULL;
 	}
@@ -705,7 +705,7 @@ static PyMethodDef LZMAFile_methods[] = {
 	{"seek", (PyCFunction)LZMAFile_seek, METH_VARARGS, LZMAFile_seek__doc__},
 	{"tell", (PyCFunction)LZMAFile_tell, METH_NOARGS, LZMAFile_tell__doc__},
 	{"close", (PyCFunction)LZMAFile_close, METH_NOARGS, LZMAFile_close__doc__},
-	{NULL,		NULL}		/* sentinel */
+	{NULL, NULL, 0, NULL}		/* sentinel */
 };
 
 
@@ -714,7 +714,7 @@ static PyMethodDef LZMAFile_methods[] = {
 
 /* This is a hacked version of Python's fileobject.c:get_newlines(). */
 static PyObject *
-LZMAFile_get_newlines(LZMAFileObject *self, void *closure)
+LZMAFile_get_newlines(LZMAFileObject *self, __attribute__((unused)) void *closure)
 {
 	switch (self->f_newlinetypes) {
 	case NEWLINE_UNKNOWN:
@@ -743,33 +743,33 @@ LZMAFile_get_newlines(LZMAFileObject *self, void *closure)
 }
 
 static PyObject *
-LZMAFile_get_closed(LZMAFileObject *self, void *closure)
+LZMAFile_get_closed(LZMAFileObject *self, __attribute__((unused)) void *closure)
 {
 	return PyInt_FromLong(!self->open);
 }
 
 static PyObject *
-LZMAFile_get_mode(LZMAFileObject *self, void *closure)
+LZMAFile_get_mode(LZMAFileObject *self, __attribute__((unused)) void *closure)
 {
 	return PyObject_GetAttrString(self->file, "mode");
 }
 
 static PyObject *
-LZMAFile_get_name(LZMAFileObject *self, void *closure)
+LZMAFile_get_name(LZMAFileObject *self, __attribute__((unused)) void *closure)
 {
 	return PyObject_GetAttrString(self->file, "name");
 }
 
 static PyGetSetDef LZMAFile_getset[] = {
 	{"closed", (getter)LZMAFile_get_closed, NULL,
-			"True if the file is closed"},
-	{"newlines", (getter)LZMAFile_get_newlines, NULL, 
-			"end-of-line convention used in this file"},
+		"True if the file is closed", NULL},
+	{"newlines", (getter)LZMAFile_get_newlines, NULL,
+		"end-of-line convention used in this file", NULL},
 	{"mode", (getter)LZMAFile_get_mode, NULL,
-			"file mode ('r', 'w', or 'U')"},
+		"file mode ('r', 'w', or 'U')", NULL},
 	{"name", (getter)LZMAFile_get_name, NULL,
-			"file name"},
-	{NULL}	/* Sentinel */
+		"file name", NULL},
+	{NULL, NULL, NULL, NULL, NULL}	/* Sentinel */
 };
 
 
@@ -782,7 +782,7 @@ static PyGetSetDef LZMAFile_getset[] = {
 static PyMemberDef LZMAFile_members[] = {
 	{"softspace",	T_INT,		OFF(f_softspace), 0,
 	 "flag indicating that a space needs to be printed; used by print"},
-	{NULL}	/* Sentinel */
+	{NULL, 0, 0, 0, NULL}	/* Sentinel */
 };
 
 /* ===================================================================== */
@@ -791,50 +791,37 @@ static PyMemberDef LZMAFile_members[] = {
 static int
 LZMAFile_init(LZMAFileObject *self, PyObject *args, PyObject *kwargs)
 {
-	PyObject *name;
+	PyObject *name = NULL, *options_dict = NULL;
 	char *mode = "r";
 	int buffering = -1;
 	lzma_ret lzuerror;
-	lzma_options_lzma *options = NULL;
+
+	static char *kwlist[] = {"name", "mode", "buffering", "memlimit",
+		"options", NULL};
+
+	self->filters[0].options = NULL;
 
 	self->size = -1;
+	self->memlimit = -1;
 
-
-	/* This is a bit cumbersome, but we don't use PyArg_ParseTupleAndKeywords
-	 * since init_lzma_options() uses this later on.
-	 */
-	if (!PyArg_ParseTuple(args, "O|si:LZMAFile",
-					&name, &mode, &buffering))
+	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O|silO:LZMAFile", kwlist,
+					&name, &mode, &buffering, &self->memlimit, &options_dict))
 		return -1;
-	if(kwargs != NULL){
-		PyObject* modeString = PyString_FromString("mode");
-		if(PyDict_Contains(kwargs, modeString)){
-			mode = PyString_AsString(PyDict_GetItem(kwargs, modeString));
-			PyDict_DelItem(kwargs, modeString);
-		}
-		PyObject* bufferingString = PyString_FromString("buffering");
-		if(PyDict_Contains(kwargs, bufferingString)){
-			buffering = PyInt_AsLong(PyDict_GetItem(kwargs, bufferingString));
-			PyDict_DelItem(kwargs, bufferingString);
-		}
-	}
 
 	for (;;) {
 		bool error = false;
 		switch (*mode) {
 			case 'w':
-				if(!options)
+				if(!self->filters[0].options)
 				{
-					lzma_options_lzma _options;
-					lzma_lzma_preset(&_options, LZMA_DEFAULT_COMPRESSION-1);
-					if(!init_lzma_options("LZMAFile", kwargs, &_options))
+					self->filters[0].options = &self->options;
+					if(!init_lzma_options("LZMAFile", options_dict, self->filters))
 						return -1;
-					options = &_options;
 				}
 				break;
 
 			case 'r':
-				if(options)
+				if(self->filters[0].options)
 					error = true;
 			case 'b':
 				break;
@@ -852,8 +839,8 @@ LZMAFile_init(LZMAFileObject *self, PyObject *args, PyObject *kwargs)
 				break;
 		}
 		if (error) {
-			if(options)
-				free(options);
+			if(self->filters[0].options)
+				free(self->filters[0].options);
 			PyErr_Format(PyExc_ValueError,
 				     "invalid mode char %c", *mode);
 			return -1;
@@ -863,7 +850,7 @@ LZMAFile_init(LZMAFileObject *self, PyObject *args, PyObject *kwargs)
 			break;
 	}
 
-	mode = options ? "wb" : "rb";
+	mode = self->filters[0].options ? "wb" : "rb";
 
 	self->file = PyObject_CallFunction((PyObject*)&PyFile_Type, "(Osi)",
 					   name, mode, buffering);
@@ -881,10 +868,10 @@ LZMAFile_init(LZMAFileObject *self, PyObject *args, PyObject *kwargs)
 	}
 #endif
 
-	self->fp = lzma_open_real(&lzuerror, options, PyFile_AsFile(self->file));
+	self->fp = lzma_open_real(&lzuerror, self->filters, PyFile_AsFile(self->file), self->memlimit);
 
 	if (lzuerror != LZMA_OK) {
-		Util_CatchLZMAError(lzuerror, &self->fp->strm);
+		Util_CatchLZMAError(lzuerror, &self->fp->strm, self->fp->encoding);
 		goto error;
 	}
 
@@ -960,10 +947,8 @@ LZMAFile_iternext(LZMAFileObject *self)
 
 PyDoc_VAR(LZMAFile__doc__) =
 PyDoc_STR(
-"LZMAFile(name [, mode='r', buffering=0, format=alone, eopm=False, level=7,\n\
-dictionary_size=23, literal_contextbits=3 literal_pos_bits=0, pos_bits=2,\n\
-mode=2, fast_bytes=128, match_finder='bt4', match_finder_cycles=0])\n\
--> file object\n\
+"LZMAFile(name [, mode='r', buffering=0, memlimit=-1,\n"
+DEFAULT_OPTIONS_STRING"]) -> file object\n\
 \n\
 Open a lzma file. The mode can be 'r' or 'w', for reading (default) or\n\
 writing. When opened for writing, the file will be created if it doesn't\n\
@@ -1023,6 +1008,16 @@ PyTypeObject LZMAFile_Type = {
         PyType_GenericNew,      		/*tp_new*/
       	_PyObject_Del,          		/*tp_free*/
         0,                      		/*tp_is_gc*/
+	0,					/*tp_bases*/
+	0,					/*tp_mro*/
+	0,					/*tp_cache*/
+	0,					/*tp_subclasses*/
+	0,					/*tp_weaklist*/
+	0,					/*tp_del*/
+#if PY_VERSION_HEX >= 0x02060000
+	0					/*tp_version_tag*/
+#endif
+
 };
 
 
